from config import cfg
from modeling.afm import AFM

import argparse
import os
import cv2
import numpy as np
import torch
import iio
import cairosvg

if __name__ == "__main__":
    
    parser = argparse.ArgumentParser(description='PyTorch Line Segment Detection (Testing)')

    parser.add_argument("--config-file",
        metavar = "FILE",
        help = "path to config file",
        type=str,
        required=True,
    )

    parser.add_argument("--gpu", type=int, default = 0)

    parser.add_argument("--epoch", dest="epoch",default=-1, type=int)


    parser.add_argument("opts",
        help="Modify config options using the command-line",
        default = None,
        nargs = argparse.REMAINDER
    )

    

    args = parser.parse_args()
    os.environ['CUDA_VISIBLE_DEVICES'] = str(args.gpu)
    cfg.merge_from_file(args.config_file)
    cfg.merge_from_list(args.opts)

    img_res = [320,320]
    im = cv2.imread('input_0.png')
    h, w, c = im.shape
    image = cv2.resize(im, (img_res[1],img_res[0]))
    image = np.array(image,dtype=np.float32)/255.0              
    image[...,0] = (image[...,0] - 0.485)/0.229
    image[...,1] = (image[...,1] - 0.456)/0.224
    image[...,2] = (image[...,2] - 0.406)/0.225
    image = np.transpose(image,(2,0,1))
    image = torch.Tensor(image).unsqueeze(0)

    system = AFM(cfg, resultDir=".")
    lines = system.test(cfg, image, args.epoch)

    scale_factor = np.array([w/img_res[1],h/img_res[0],w/img_res[1],h/img_res[0]],dtype=np.float32)    
    lines[:,:4] *=scale_factor

    ##### OUTPUT ######

    out_svg = 'lines.svg'
    out_txt = 'lines.txt'
    out_png = 'lines.png'

    ftxt = open(out_txt, "w")
    fcsv = open(out_svg, "w")

    fcsv.write('<?xml version="1.0" standalone="no"?>\n<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"\n"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg version="1.1" xmlns="http://www.w3.org/2000/svg"\nxmlns:xlink="http://www.w3.org/1999/xlink"\nwidth="'+str(w)+'" height="'+str(h)+'" viewBox="0 0 '+str(w)+' '+str(h)+'">')
    topng = '<?xml version="1.0" standalone="no"?>\n<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN"\n"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd"><svg version="1.1" xmlns="http://www.w3.org/2000/svg"\nxmlns:xlink="http://www.w3.org/1999/xlink"\nwidth="'+str(w)+'" height="'+str(h)+'" viewBox="0 0 '+str(w)+' '+str(h)+'">'

    color='black'
    width=1.5
    for tp_id, line in enumerate(lines):
        x1, y1, x2, y2, _ = line # this is yxyx
        fcsv.write('<line x1="'+str(x1)+'" y1="' +str(y1)+ '" x2="' +str(x2)+ '" y2="' +str(y2)+ '" stroke="'+color+'" stroke-width="'+str(width)+'"/>\n')
        topng += '<line x1="'+str(x1)+'" y1="' +str(y1)+ '" x2="' +str(x2)+ '" y2="' +str(y2)+ '" stroke="white" stroke-width="'+str(width)+'"/>\n'
        ftxt.write(str(x1)+' '+str(y1)+' '+str(x2)+' '+str(y2)+'\n')
    fcsv.write('\n</svg>')
    topng += '\n</svg>'
    ftxt.close() 
    fcsv.close() 

    cairosvg.svg2png(bytestring=topng, write_to=out_png)
    im = iio.read(out_png)
    iio.write(out_png, im[:,:,:3])

 
